% Example - Extrapolation: fixed point iteration on linear function
%   solve the equation x = Ax + b for random A and b

d = 10; n=2*d;
A = rand(d); b = rand(d,1); x_true = -(A - eye(d)) \ b;
X = b;
for i=1:n
    X(:,i+1) = A*X(:,i) + b;
end
F = X(:,2:end) - X(:,1:n);

[x_v1,r_v1] = ALGO_extrap_MPE_v1_20211022(F,X(:,1:n));
[x_v2,r_v2] = ALGO_extrap_MPE_v2_20211029(F,X(:,1:n));
x = x_v1(:,end);
Error_abs= norm(x - x_true)
Error_rel= Error_abs/norm(x_true)
Residual = norm(A*x + b - x)

Error_v1 = zeros(1,n); res_MPE_v1 = zeros(1,n);
Error_v2 = zeros(1,n); res_MPE_v2 = zeros(1,n);
for i=1:n
    Error_v1(i) = norm(x_v1(:,i) - x_true);
    Error_v2(i) = norm(x_v2(:,i) - x_true);
    res_MPE_v1(i) = norm(A*x_v1(:,i) + b - x_v1(:,i));
    res_MPE_v2(i) = norm(A*x_v2(:,i) + b - x_v2(:,i));
end

[x_GMRES,~,~,~,res_GMRES]=gmres(A-eye(d),-b,d,0,n,eye(d),eye(d),X(:,1));

figure(1)
semilogy(1:n,Error_v1,'b*--',1:n,Error_v2,'k.--')
xlabel('Iteration')
ylabel('Error in solution')
title('Error vs iteration')
legend('MPE w/ Arnoldi','MPE','GMRES')

figure(2)
semilogy(1:n,res_MPE_v1,'b*--',1:n,res_MPE_v2,'k.--',1:length(res_GMRES),res_GMRES,'ro')
xlabel('Iteration')
ylabel('Residual')
title('Residual vs iteration')
legend('MPE w/ Arnoldi','MPE','GMRES')